/****************************************************************************
Module name: VMAlloc.C
*****************************************************************************/

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>    // for sprintf
#include "VMAlloc.H"


const UINT g_uPageSize = 4 * 1024;

typedef struct {
   BOOL fAllocated;
   BYTE bOtherData[2048 - sizeof(BOOL)];
} SOMEDATA, *PSOMEDATA;

#define MAX_SOMEDATA  (50)
PSOMEDATA g_pSomeData = NULL;

RECT g_rcMemMap;


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus, LPARAM lParam) {
   // Initialize the dialog box by disabling all of the non-setup controls.
   EnableWindow(GetDlgItem(hwnd, ID_INDEXTEXT), FALSE);
   EnableWindow(GetDlgItem(hwnd, ID_INDEX), FALSE);
   ScrollBar_SetRange(GetDlgItem(hwnd, ID_INDEXSCRL), 0, MAX_SOMEDATA - 1,
       FALSE);
   ScrollBar_SetPos(GetDlgItem(hwnd, ID_INDEXSCRL), 0,TRUE);
   EnableWindow(GetDlgItem(hwnd, ID_INDEXSCRL), FALSE);
   EnableWindow(GetDlgItem(hwnd, ID_USE), FALSE);
   EnableWindow(GetDlgItem(hwnd, ID_CLEAR), FALSE);
   EnableWindow(GetDlgItem(hwnd, ID_GARBAGECOLLECT), FALSE);

   // Get the coordinates of the memory map display.
   GetWindowRect(GetDlgItem(hwnd, ID_MEMMAP), &g_rcMemMap);
   MapWindowPoints(NULL, hwnd, (LPPOINT) &g_rcMemMap, 2);

   // Destroy the temporary window that identifies the location
   // of the memory map display.
   DestroyWindow(GetDlgItem(hwnd, ID_MEMMAP));

   return(TRUE);
}


void Dlg_OnDestroy (HWND hwnd) {
   if (g_pSomeData != NULL)
      VirtualFree(g_pSomeData, 0, MEM_RELEASE);
}

void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl, UINT codeNotify) {
   UINT uIndex, uIndexLast, uPage, uMaxPages;
   BOOL fTranslated, fOk, fAnyAllocs;
   MEMORY_BASIC_INFORMATION MemoryBasicInfo;

   switch (id) {
      case ID_RESERVE:
         // Reserve enough address space to hold MAX_SOMEDATA
         // SOMEDATA structures.

         g_pSomeData = VirtualAlloc(NULL, MAX_SOMEDATA * sizeof(SOMEDATA),
            MEM_RESERVE, PAGE_NOACCESS);

         // Disable the Reserve button and enable all of the other controls.
         EnableWindow(GetDlgItem(hwnd, ID_RESERVE), FALSE);

         EnableWindow(GetDlgItem(hwnd, ID_INDEXTEXT), TRUE);
         EnableWindow(GetDlgItem(hwnd, ID_INDEX), TRUE);
         EnableWindow(GetDlgItem(hwnd, ID_INDEXSCRL), TRUE);
         EnableWindow(GetDlgItem(hwnd, ID_USE), TRUE);
         EnableWindow(GetDlgItem(hwnd, ID_GARBAGECOLLECT), TRUE);

         // Invalidate the memory map display.
         InvalidateRect(hwnd, &g_rcMemMap, FALSE);

         break;

      case ID_INDEX:
         if (codeNotify != EN_CHANGE) break;
         uIndex = GetDlgItemInt(hwnd, id, &fTranslated, FALSE);
         if (uIndex >= MAX_SOMEDATA) {
            // If the index is out of range, assume that the
            // tranlation was unsuccessful.
            fTranslated = FALSE;
         }

         if (fTranslated) {
            VirtualQuery(&g_pSomeData[uIndex], &MemoryBasicInfo,
               sizeof(MemoryBasicInfo));
            fOk = MemoryBasicInfo.State == MEM_COMMIT;
            if (fOk) fOk = fOk && g_pSomeData[uIndex].fAllocated;
            EnableWindow(GetDlgItem(hwnd, ID_USE), !fOk);
            EnableWindow(GetDlgItem(hwnd, ID_CLEAR), fOk);
            ScrollBar_SetPos(GetDlgItem(hwnd, ID_INDEXSCRL), uIndex, TRUE);

         } else {

            EnableWindow(GetDlgItem(hwnd, ID_USE), FALSE);
            EnableWindow(GetDlgItem(hwnd, ID_CLEAR), FALSE);
         }
         break;

      case ID_USE:
         uIndex = GetDlgItemInt(hwnd, ID_INDEX, &fTranslated, FALSE);
         VirtualAlloc(&g_pSomeData[uIndex], sizeof(SOMEDATA), MEM_COMMIT,
            PAGE_READWRITE);

         // When pages are committed, NT makes sure that they are zeroed.
         g_pSomeData[uIndex].fAllocated = TRUE;

         EnableWindow(GetDlgItem(hwnd, ID_USE), FALSE);
         EnableWindow(GetDlgItem(hwnd, ID_CLEAR), TRUE);

         // Invalidate the memory map display.
         InvalidateRect(hwnd, &g_rcMemMap, FALSE);

         break;

      case ID_CLEAR:
         uIndex = GetDlgItemInt(hwnd, ID_INDEX, &fTranslated, FALSE);
         g_pSomeData[uIndex].fAllocated = FALSE;
         EnableWindow(GetDlgItem(hwnd, ID_USE), TRUE);
         EnableWindow(GetDlgItem(hwnd, ID_CLEAR), FALSE);
         break;

      case ID_GARBAGECOLLECT:
         uMaxPages = MAX_SOMEDATA * sizeof(SOMEDATA) / g_uPageSize;
         for (uPage = 0; uPage < uMaxPages; uPage++) {
            fAnyAllocs = FALSE;
            uIndex = uPage * g_uPageSize  / sizeof(SOMEDATA);
            uIndexLast = uIndex + g_uPageSize / sizeof(SOMEDATA);
            for (; uIndex < uIndexLast; uIndex++) {
               VirtualQuery(&g_pSomeData[uIndex], &MemoryBasicInfo,
                  sizeof(MemoryBasicInfo));
               if ((MemoryBasicInfo.State == MEM_COMMIT) &&
                  g_pSomeData[uIndex].fAllocated) {
                  fAnyAllocs = TRUE;
                  break;
               }
            }

            if (!fAnyAllocs) {
               // No allocated structures exist in the page, decommit it.
               VirtualFree(&g_pSomeData[uIndexLast - 1], sizeof(SOMEDATA),
                  MEM_DECOMMIT);
            }
         }

         // Invalidate the memory map display.
         InvalidateRect(hwnd, &g_rcMemMap, FALSE);
         break;

      case IDCANCEL:
         // Decommit and free the virtual address space.
         EndDialog(hwnd, id); break;
   }
}

void Dlg_OnHScroll (HWND hwnd, HWND hwndCtl, UINT code, int pos) {
   INT nScrlPos;
   if (hwndCtl != GetDlgItem(hwnd, ID_INDEXSCRL))
      return;

   nScrlPos = ScrollBar_GetPos(hwndCtl);
   switch (code) {
      case SB_LINELEFT:
         nScrlPos--;
         break;

      case SB_LINERIGHT:
         nScrlPos++;
         break;

      case SB_PAGELEFT:
         nScrlPos -= g_uPageSize / sizeof(SOMEDATA);
         break;

      case SB_PAGERIGHT:
         nScrlPos += g_uPageSize / sizeof(SOMEDATA);
         break;

      case SB_THUMBTRACK:
         nScrlPos = pos;
         break;

      case SB_LEFT:
         nScrlPos = 0;
         break;

      case SB_RIGHT:
         nScrlPos = MAX_SOMEDATA - 1;
         break;
   }
   if (nScrlPos < 0)
      nScrlPos = 0;

   if (nScrlPos >= MAX_SOMEDATA)
      nScrlPos = MAX_SOMEDATA - 1;

   ScrollBar_SetPos(hwndCtl, nScrlPos, TRUE);
   SetDlgItemInt(hwnd, ID_INDEX, nScrlPos, TRUE);
}



void Dlg_OnPaint (HWND hwnd) {
   UINT uPage, uIndex, uIndexLast, uMemMapWidth;
   UINT uMaxPages = MAX_SOMEDATA * sizeof(SOMEDATA) / g_uPageSize;
   BOOL fAnyAllocs;
   MEMORY_BASIC_INFORMATION MemoryBasicInfo;
   PAINTSTRUCT ps;

   BeginPaint(hwnd, &ps);

   if (g_pSomeData == NULL) {
      // The memory has yet to be reserved.
      Rectangle(ps.hdc, g_rcMemMap.left, g_rcMemMap.top,
         g_rcMemMap.right, g_rcMemMap.bottom);
   }

   // Walk the virtual address space adding entries to the ListBox.
   for (uPage = 0; (g_pSomeData != NULL) && uPage < uMaxPages; uPage++) {
      fAnyAllocs = FALSE;
      uIndex = uPage * g_uPageSize / sizeof(SOMEDATA);
      uIndexLast = uIndex + g_uPageSize / sizeof(SOMEDATA);
      for (; uIndex < uIndexLast; uIndex++) {

         VirtualQuery(&g_pSomeData[uIndex], &MemoryBasicInfo,
            sizeof(MemoryBasicInfo));

         switch (MemoryBasicInfo.State) {
            case MEM_FREE:
               SelectObject(ps.hdc, GetStockObject(WHITE_BRUSH));
               break;

            case MEM_RESERVE:
               SelectObject(ps.hdc, GetStockObject(GRAY_BRUSH));
               break;

            case MEM_COMMIT:
               SelectObject(ps.hdc, GetStockObject(BLACK_BRUSH));
               break;
         }

         uMemMapWidth = g_rcMemMap.right - g_rcMemMap.left;
         Rectangle(ps.hdc,
            g_rcMemMap.left + uMemMapWidth / uMaxPages * uPage,
            g_rcMemMap.top,
            g_rcMemMap.left + uMemMapWidth / uMaxPages * (uPage + 1),
            g_rcMemMap.bottom);
      }
   }
   EndPaint(hwnd, &ps);
}


BOOL CALLBACK DlgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   BOOL fProcessed = TRUE;
   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);
      HANDLE_MSG(hDlg, WM_HSCROLL, Dlg_OnHScroll);
      HANDLE_MSG(hDlg, WM_PAINT, Dlg_OnPaint);
      HANDLE_MSG(hDlg, WM_DESTROY, Dlg_OnDestroy);

      default: fProcessed = FALSE; break;
   }
   return(fProcessed);
}


int APIENTRY WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
   LPSTR lpszCmdLine, int nCmdShow) {

   DialogBox(hInstance, MAKEINTRESOURCE(DLG_VMALLOC), NULL, DlgProc);
   return(0);
}

